package jdk.internal.org.objectweb.asm.tree.analysis;

import java.util.List;
import jdk.internal.org.objectweb.asm.Type;

public class SimpleVerifier extends BasicVerifier
{
    private final Type currentClass;
    private final Type currentSuperClass;
    private final List<Type> currentClassInterfaces;
    private final boolean isInterface;
    private ClassLoader loader;
    
    public SimpleVerifier() {
        this(null, null, false);
    }
    
    public SimpleVerifier(Type type, Type type2, boolean b) {
        this(type, type2, null, b);
    }
    
    public SimpleVerifier(Type type, Type type2, List<Type> list, boolean b) {
        this(327680, type, type2, list, b);
    }
    
    protected SimpleVerifier(int n, Type currentClass, Type currentSuperClass, List<Type> currentClassInterfaces, boolean isInterface) {
        super(n);
        this.loader = this.getClass().getClassLoader();
        this.currentClass = currentClass;
        this.currentSuperClass = currentSuperClass;
        this.currentClassInterfaces = currentClassInterfaces;
        this.isInterface = isInterface;
    }
    
    public void setClassLoader(ClassLoader loader) {
        this.loader = loader;
    }
    
    @Override
    public BasicValue newValue(Type type) {
        if (type == null) {
            return BasicValue.UNINITIALIZED_VALUE;
        }
        int n = type.getSort() == 9;
        if (!(n == 0)) {
            switch (type.getElementType().getSort()) {
                case 1:
                case 2:
                case 3:
                case 4: {
                    return new BasicValue(type);
                }
                default: {
                    break;
                }
            }
        }
        BasicValue value = super.newValue(type);
        if (BasicValue.REFERENCE_VALUE.equals(value)) {
            if (!(n == 0)) {
                String s = this.newValue(type.getElementType()).getType().getDescriptor();
                for (int i = 0; i < type.getDimensions(); ++i) {
                    s = 91 + s;
                }
                value = new BasicValue(Type.getType(s));
            }
            else {
                value = new BasicValue(type);
            }
        }
        return value;
    }
    
    @Override
    protected boolean isArrayValue(BasicValue basicValue) {
        Type type = basicValue.getType();
        return type != null && ("Lnull;".equals(type.getDescriptor()) || type.getSort() == 9);
    }
    
    @Override
    protected BasicValue getElementValue(BasicValue basicValue) throws AnalyzerException {
        Type type = basicValue.getType();
        if (type != null) {
            if (type.getSort() == 9) {
                return this.newValue(Type.getType(type.getDescriptor().substring(1)));
            }
            if ("Lnull;".equals(type.getDescriptor())) {
                return basicValue;
            }
        }
        throw new Error("Internal error");
    }
    
    @Override
    protected boolean isSubTypeOf(BasicValue basicValue, BasicValue basicValue2) {
        Type type = basicValue2.getType();
        Type type2 = basicValue.getType();
        switch (type.getSort()) {
            case 5:
            case 6:
            case 7:
            case 8: {
                return type2.equals(type);
            }
            case 9:
            case 10: {
                return "Lnull;".equals(type2.getDescriptor()) || ((type2.getSort() == 10 || type2.getSort() == 9) && this.isAssignableFrom(type, type2));
            }
            default: {
                throw new Error("Internal error");
            }
        }
    }
    
    @Override
    public BasicValue merge(BasicValue basicValue, BasicValue basicValue2) {
        if (!basicValue.equals(basicValue2)) {
            Type type = basicValue.getType();
            Type type2 = basicValue2.getType();
            if (type != null && (type.getSort() == 10 || type.getSort() == 9) && type2 != null && (type2.getSort() == 10 || type2.getSort() == 9)) {
                if ("Lnull;".equals(type.getDescriptor())) {
                    return basicValue2;
                }
                if ("Lnull;".equals(type2.getDescriptor())) {
                    return basicValue;
                }
                if (this.isAssignableFrom(type, type2)) {
                    return basicValue;
                }
                if (this.isAssignableFrom(type2, type)) {
                    return basicValue2;
                }
                while (type != null && !this.isInterface(type)) {
                    type = this.getSuperClass(type);
                    if (!this.isAssignableFrom(type, type2)) {
                        continue;
                    }
                    return this.newValue(type);
                }
                return BasicValue.REFERENCE_VALUE;
            }
            else {
                return BasicValue.UNINITIALIZED_VALUE;
            }
        }
        else {
            return basicValue;
        }
    }
    
    protected boolean isInterface(Type type) {
        if (this.currentClass != null && type.equals(this.currentClass)) {
            return this.isInterface;
        }
        return this.getClass(type).isInterface();
    }
    
    protected Type getSuperClass(Type type) {
        if (this.currentClass != null && type.equals(this.currentClass)) {
            return this.currentSuperClass;
        }
        Class<Object> superclass = this.getClass(type).getSuperclass();
        return (superclass == null) ? null : Type.getType(superclass);
    }
    
    protected boolean isAssignableFrom(Type type, Type object) {
        if (type.equals(object)) {
            return true;
        }
        if (this.currentClass != null && type.equals(this.currentClass)) {
            if (this.getSuperClass(object) == null) {
                return false;
            }
            if (this.isInterface) {
                return object.getSort() == 10 || object.getSort() == 9;
            }
            return this.isAssignableFrom(type, this.getSuperClass(object));
        }
        else if (this.currentClass != null && object.equals(this.currentClass)) {
            if (this.isAssignableFrom(type, this.currentSuperClass)) {
                return true;
            }
            if (this.currentClassInterfaces != null) {
                int i = 0;
                while (i < this.currentClassInterfaces.size()) {
                    if (this.isAssignableFrom(type, (Type)this.currentClassInterfaces.get(i))) {
                        return true;
                    }
                    ++i;
                }
            }
            return false;
        }
        else {
            Class<Object> class1 = this.getClass(type);
            if (class1.isInterface()) {
                class1 = Object.class;
            }
            return class1.isAssignableFrom(this.getClass(object));
        }
    }
    
    protected Class<?> getClass(Type type) {
        try {
            if (type.getSort() == 9) {
                return Class.forName(type.getDescriptor().replace('/', '.'), false, this.loader);
            }
            return Class.forName(type.getClassName(), false, this.loader);
        }
        catch (ClassNotFoundException ex) {
            throw new RuntimeException(ex.toString());
        }
    }
}
